/*
    Copyright (c) 2019 David Faure <faure@kde.org>

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#ifndef AKONADI_DBDEADLOCKCATCHER_H
#define AKONADI_DBDEADLOCKCATCHER_H

#include "dbexception.h"

namespace Akonadi
{

namespace Server
{

/**
  This class catches DbDeadlockException (as emitted by QueryBuilder)
  and retries execution of the method when it happens, as required by
  SQL databases.
*/
class DbDeadlockCatcher
{
public:
    template <typename Func>
    explicit DbDeadlockCatcher(Func &&func) { callFunc(func, 0); }

private:
    static const int MaxRecursion = 5;
    template <typename Func>
    void callFunc(Func &&func, int recursionCounter) {
        try {
            func();
        } catch(const DbDeadlockException &) {
            if (recursionCounter == MaxRecursion) {
                throw;
             } else {
                callFunc(func, ++recursionCounter); // recurse
            }
        }
    }
};

} // namespace Server
} // namespace Akonadi

#endif
